@@ -1,33 +1,24 @@
-# pylint: disable=W0611
-#
-# Copyright (c) 2003-2013 LOGILAB S.A. (Paris, FRANCE).
+# Copyright (c) 2003-2016 LOGILAB S.A. (Paris, FRANCE).
 # http://www.logilab.fr/ -- mailto:contact@logilab.fr
-#
-# This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any later
-# version.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+# For details: https://github.com/PyCQA/pylint/blob/master/COPYING
+
+# pylint: disable=W0611
 """some functions that may be useful for various checkers
 """
 import functools
+import itertools
 import re
 import sys
 import string
 import warnings
 
-import astroid
-from astroid import helpers
-from astroid import scoped_nodes
+import six
 from six.moves import map, builtins # pylint: disable=redefined-builtin
 
+import astroid
+from astroid import scoped_nodes
+
 BUILTINS_NAME = builtins.__name__
 COMP_NODE_TYPES = (astroid.ListComp, astroid.SetComp,
                    astroid.DictComp, astroid.GeneratorExp)
@@ -39,6 +30,11 @@ else:
     EXCEPTIONS_MODULE = "builtins"
 ABC_METHODS = set(('abc.abstractproperty', 'abc.abstractmethod',
                    'abc.abstractclassmethod', 'abc.abstractstaticmethod'))
+ITER_METHOD = '__iter__'
+NEXT_METHOD = 'next' if six.PY2 else '__next__'
+GETITEM_METHOD = '__getitem__'
+CONTAINS_METHOD = '__contains__'
+KEYS_METHOD = 'keys'
 
 # Dictionary which maps the number of expected parameters a
 # special method can have to a set of special methods.
@@ -73,7 +69,7 @@ _SPECIAL_METHODS_PARAMS = {
         '__itruediv__', '__ifloordiv__', '__imod__', '__ilshift__',
         '__irshift__', '__iand__', '__ixor__', '__ior__', '__ipow__',
         '__setstate__', '__reduce_ex__', '__deepcopy__', '__cmp__',
-        '__matmul__', '__rmatmul__'),
+        '__matmul__', '__rmatmul__', '__div__'),
 
     2: ('__setattr__', '__get__', '__set__', '__setitem__'),
 
@@ -377,7 +373,7 @@ def is_attr_protected(attrname):
     """return True if attribute name is protected (start with _ and some other
     details), False otherwise.
     """
-    return attrname[0] == '_' and not attrname == '_' and not (
+    return attrname[0] == '_' and attrname != '_' and not (
         attrname.startswith('__') and attrname.endswith('__'))
 
 def node_frame_class(node):
@@ -404,12 +400,11 @@ def is_attr_private(attrname):
 
 def get_argument_from_call(callfunc_node, position=None, keyword=None):
     """Returns the specified argument from a function call.
-
-    :param callfunc_node: Node representing a function call to check.
+    :param astroid.Call callfunc_node: Node representing a function call to check.
     :param int position: position of the argument.
     :param str keyword: the keyword of the argument.
-
     :returns: The node representing the argument, None if the argument is not found.
+    :rtype: astroid.Name
     :raises ValueError: if both position and keyword are None.
     :raises NoSuchArgumentError: if no argument at the provided position or with
     the provided keyword.
@@ -437,7 +432,7 @@ def inherit_from_std_ex(node):
             and node.root().name == EXCEPTIONS_MODULE:
         return True
     return any(inherit_from_std_ex(parent)
-               for parent in node.ancestors(recurs=False))
+               for parent in node.ancestors(recurs=True))
 
 def error_of_type(handler, error_type):
     """
@@ -445,24 +440,26 @@ def error_of_type(handler, error_type):
     the given error_type.
 
     The *handler* parameter is a node, representing an ExceptHandler node.
-    The *error_type* can be an exception, such as AttributeError, or it
-    can be a tuple of errors.
+    The *error_type* can be an exception, such as AttributeError,
+    the name of an exception, or it can be a tuple of errors.
     The function will return True if the handler catches any of the
     given errors.
     """
+    def stringify_error(error):
+        if not isinstance(error, six.string_types):
+            return error.__name__
+        return error
+
     if not isinstance(error_type, tuple):
         error_type = (error_type, )
-    expected_errors = {error.__name__ for error in error_type}
+    expected_errors = {stringify_error(error) for error in error_type}
+    if not handler.type:
+        # bare except. While this indeed catches anything, if the desired errors
+        # aren't specified directly, then we just ignore it.
+        return False
     return handler.catch(expected_errors)
 
 
-def is_import_error(handler):
-    warnings.warn("This function is deprecated in the favour of "
-                  "error_of_type. It will be removed in Pylint 1.6.",
-                  DeprecationWarning, stacklevel=2)
-    return error_of_type(handler, ImportError)
-
-
 def decorated_with_property(node):
     """ Detect if the given function node is decorated with a property. """
     if not node.decorators:
@@ -551,28 +548,215 @@ def unimplemented_abstract_methods(node, is_abstract_cb=None):
     return visited
 
 
-def node_ignores_exception(node, exception):
-    """Check if the node is in a TryExcept which handles the given exception.
-
-    This will also return ``True`` if the node is protected by an `except Exception`
-    or by a bare except clause.
-    """
+def _import_node_context(node):
     current = node
     ignores = (astroid.ExceptHandler, astroid.TryExcept)
     while current and not isinstance(current.parent, ignores):
         current = current.parent
 
+    if current and isinstance(current.parent, ignores):
+        return current.parent
+    return None
+
+
+def is_from_fallback_block(node):
+    """Check if the given node is from a fallback import block."""
+    context = _import_node_context(node)
+    if not context:
+        return False
+
+    if isinstance(context, astroid.ExceptHandler):
+        other_body = context.parent.body
+        handlers = context.parent.handlers
+    else:
+        other_body = itertools.chain.from_iterable(
+            handler.body for handler in context.handlers)
+        handlers = context.handlers
+
+    has_fallback_imports = any(isinstance(import_node, (astroid.ImportFrom, astroid.Import))
+                               for import_node in other_body)
+    ignores_import_error = _except_handlers_ignores_exception(handlers, ImportError)
+    return ignores_import_error or has_fallback_imports
+
+
+def _except_handlers_ignores_exception(handlers, exception):
     func = functools.partial(error_of_type,
-                             error_type=(Exception, exception))
+                             error_type=(exception, ))
+    return any(map(func, handlers))
+
+
+def node_ignores_exception(node, exception):
+    """Check if the node is in a TryExcept which handles the given exception."""
+    current = node
+    ignores = (astroid.ExceptHandler, astroid.TryExcept)
+    while current and not isinstance(current.parent, ignores):
+        current = current.parent
+
     if current and isinstance(current.parent, astroid.TryExcept):
-        handles_errors = any(map(func, current.parent.handlers))
-        empty_handlers = any(handler.type is None
-                             for handler in current.parent.handlers)
-        if handles_errors or empty_handlers:
+        return _except_handlers_ignores_exception(current.parent.handlers, exception)
+    return False
+
+
+def class_is_abstract(node):
+    """return true if the given class node should be considered as an abstract
+    class
+    """
+    for method in node.methods():
+        if method.parent.frame() is node:
+            if method.is_abstract(pass_is_abstract=False):
+                return True
+    return False
+
+
+def _hasattr(value, attr):
+    try:
+        value.getattr(attr)
+        return True
+    except astroid.NotFoundError:
+        return False
+
+
+def is_comprehension(node):
+    comprehensions = (astroid.ListComp,
+                      astroid.SetComp,
+                      astroid.DictComp,
+                      astroid.GeneratorExp)
+    return isinstance(node, comprehensions)
+
+
+def _supports_mapping_protocol(value):
+    return _hasattr(value, GETITEM_METHOD) and _hasattr(value, KEYS_METHOD)
+
+
+def _supports_membership_test_protocol(value):
+    return _hasattr(value, CONTAINS_METHOD)
+
+
+def _supports_iteration_protocol(value):
+    return _hasattr(value, ITER_METHOD) or _hasattr(value, GETITEM_METHOD)
+
+
+def _supports_subscript_protocol(value):
+    return _hasattr(value, GETITEM_METHOD)
+
+
+def _is_abstract_class_name(name):
+    lname = name.lower()
+    is_mixin = lname.endswith('mixin')
+    is_abstract = lname.startswith('abstract')
+    is_base = lname.startswith('base') or lname.endswith('base')
+    return is_mixin or is_abstract or is_base
+
+
+def is_inside_abstract_class(node):
+    while node is not None:
+        if isinstance(node, astroid.ClassDef):
+            if class_is_abstract(node):
+                return True
+            name = getattr(node, 'name', None)
+            if name is not None and _is_abstract_class_name(name):
+                return True
+        node = node.parent
+    return False
+
+
+def is_iterable(value):
+    if isinstance(value, astroid.ClassDef):
+        if not has_known_bases(value):
+            return True
+        # classobj can only be iterable if it has an iterable metaclass
+        meta = value.metaclass()
+        if meta is not None:
+            if _supports_iteration_protocol(meta):
+                return True
+    if isinstance(value, astroid.Instance):
+        if not has_known_bases(value):
+            return True
+        if _supports_iteration_protocol(value):
             return True
     return False
 
 
+def is_mapping(value):
+    if isinstance(value, astroid.ClassDef):
+        if not has_known_bases(value):
+            return True
+        # classobj can only be a mapping if it has a metaclass is mapping
+        meta = value.metaclass()
+        if meta is not None:
+            if _supports_mapping_protocol(meta):
+                return True
+    if isinstance(value, astroid.Instance):
+        if not has_known_bases(value):
+            return True
+        if _supports_mapping_protocol(value):
+            return True
+    return False
+
+
+def supports_membership_test(value):
+    if isinstance(value, astroid.ClassDef):
+        if not has_known_bases(value):
+            return True
+        meta = value.metaclass()
+        if meta is not None and _supports_membership_test_protocol(meta):
+            return True
+    if isinstance(value, astroid.Instance):
+        if not has_known_bases(value):
+            return True
+        if _supports_membership_test_protocol(value):
+            return True
+    return is_iterable(value)
+
+
+def supports_subscript(value):
+    if isinstance(value, astroid.ClassDef):
+        if not has_known_bases(value):
+            return True
+        meta = value.metaclass()
+        if meta is not None and _supports_subscript_protocol(meta):
+            return True
+    if isinstance(value, astroid.Instance):
+        if not has_known_bases(value):
+            return True
+        if _supports_subscript_protocol(value):
+            return True
+    return False
+
 # TODO(cpopa): deprecate these or leave them as aliases?
-safe_infer = astroid.helpers.safe_infer
-has_known_bases = astroid.helpers.has_known_bases
+def safe_infer(node, context=None):
+    """Return the inferred value for the given node.
+
+    Return None if inference failed or if there is some ambiguity (more than
+    one node has been inferred).
+    """
+    try:
+        inferit = node.infer(context=context)
+        value = next(inferit)
+    except astroid.InferenceError:
+        return
+    try:
+        next(inferit)
+        return # None if there is ambiguity on the inferred node
+    except astroid.InferenceError:
+        return # there is some kind of ambiguity
+    except StopIteration:
+        return value
+
+
+def has_known_bases(klass, context=None):
+    """Return true if all base classes of a class could be inferred."""
+    try:
+        return klass._all_bases_known
+    except AttributeError:
+        pass
+    for base in klass.bases:
+        result = safe_infer(base, context=context)
+        # TODO: check for A->B->A->B pattern in class structure too?
+        if (not isinstance(result, astroid.ClassDef) or
+                result is klass or
+                not has_known_bases(result, context=context)):
+            klass._all_bases_known = False
+            return False
+    klass._all_bases_known = True
+    return True
