@@ -19,7 +19,7 @@ xml.etree.ElementTree.Element('tag').getiterator()  # [deprecated-method]
 xml.etree.ElementTree.XMLParser('tag', None, None).doctype(None, None, None)  # [deprecated-method]
 nntplib.NNTP(None).xpath(None) # [deprecated-method]
 inspect.getmoduleinfo(inspect) # [deprecated-method]
-inspect.getmodulename(inspect) # [deprecated-method]
+
 inspect.getargspec(None) # [deprecated-method]
 logging.warn("a") # [deprecated-method]
 platform.popen([]) # [deprecated-method]
@@ -33,3 +33,5 @@ class SuperCrash(unittest.TestCase):
     def __init__(self):
         # should not crash.
         super(SuperCrash, self)()
+
+xml.etree.ElementTree.iterparse(None)
